/*
 *  Licensed to the Apache Software Foundation (ASF) under one or more
 *  contributor license agreements.  See the NOTICE file distributed with
 *  this work for additional information regarding copyright ownership.
 *  The ASF licenses this file to You under the Apache License, Version 2.0
 *  (the "License"); you may not use this file except in compliance with
 *  the License.  You may obtain a copy of the License at
 *
 *      https://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

package org.apache.tools.ant.taskdefs.condition;

import org.junit.Before;
import org.junit.Test;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

/**
 * Testcase for the &lt;equals&gt; condition.
 *
 */
public class EqualsTest {

    private Equals eq;

    @Before
    public void setUp() {
        eq = new Equals();
    }

    @Test
    public void testTrim() {
        eq.setArg1("a");
        eq.setArg2(" a");
        assertFalse(eq.eval());

        eq.setTrim(true);
        assertTrue(eq.eval());

        eq.setArg2("a\t");
        assertTrue(eq.eval());
    }

    @Test
    public void testCaseSensitive() {
        eq.setArg1("a");
        eq.setArg2("A");
        assertFalse(eq.eval());

        eq.setCasesensitive(false);
        assertTrue(eq.eval());
    }

}
