/*
 *  Licensed to the Apache Software Foundation (ASF) under one or more
 *  contributor license agreements.  See the NOTICE file distributed with
 *  this work for additional information regarding copyright ownership.
 *  The ASF licenses this file to You under the Apache License, Version 2.0
 *  (the "License"); you may not use this file except in compliance with
 *  the License.  You may obtain a copy of the License at
 *
 *      https://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/**
 * The classes/interfaces in this package <em>must not</em>
 * have any compile time dependency on any of JUnit platform or
 * engine classes/interfaces. They <em>must not</em> even have any
 * compile time dependency on any classes/interfaces that belong to the
 * {@code org.apache.tools.ant.taskdefs.optional.junitlauncher} package.
 *
 * @since Ant 1.10.6
 */
package org.apache.tools.ant.taskdefs.optional.junitlauncher.confined;
